/*
           Copyright Devin DeLong 2006 - 2007.
  Distributed under the Boost Software License, Version 1.0.
     (See accompanying file LICENSE_1_0.txt or copy at
           http://www.boost.org/LICENSE_1_0.txt)
 */

// BZFS
#include "bzfsAPI.h"

// Local
#include "EventHandler.h"
#include "Mine.h"
#include "utility.h"
#include "Global.h"

// STL
#include <algorithm>
#include <functional>

// Boost
#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/bind.hpp>


void EventHandler::process(bz_EventData *eventData)
{
  switch(eventData->eventType) {
        
        
    ////////////////////////// Update Event ///////////////////////////////////////////////////// 
    
    /** Where the magic happens */
      
    case bz_ePlayerUpdateEvent:
    {
            
      bz_PlayerUpdateEventData *eData = 
        dynamic_cast<bz_PlayerUpdateEventData*>(eventData);
     
      float triggerRadius = 0.9f * float(bz_getBZDBDouble("_shockOutRadius"));
      
      boost::ptr_vector<Mine>::iterator mine = Global::mineList.begin();
      boost::ptr_vector<Mine>::iterator end = Global::mineList.end();
      
      while(mine != end) {
        
        /* Always ignore the player who set the mine */
        
        if(eData->playerID == mine->playerID()) {
          ++mine; 
          continue;
        }
                
        /* Ignore players who are not close enough */
        
        if(distance3f(eData->pos, mine->position().data()) > triggerRadius) {
          ++mine; 
          continue;
        }
        
        if(Global::teamKillSafety) {
          bz_PlayerRecord *player = bz_getPlayerByIndex(eData->playerID);
          if(!player) { ++mine; continue; }
          
          bz_PlayerRecord *killer = bz_getPlayerByIndex(mine->playerID());
          if(!killer) { bz_freePlayerRecord(player); ++mine; continue; }        
          
          // Ignore teammates?
          if(player->team == killer->team &&
             player->team != eRogueTeam) {
            bz_freePlayerRecord(killer);
            bz_freePlayerRecord(player);          
            ++mine; 
            continue;
          }
          
          bz_freePlayerRecord(killer);
          bz_freePlayerRecord(player);
        }
        
        // Not good to have Mine logic over here
        if(mine->type() == eBadFlagMine) {
          giveBadFlag(eData->playerID);
          bz_sendTextMessage(eData->playerID, eData->playerID, 
                             "You stepped on a bad flag mine"); 
        }

        mine->fire();
        lastKillerID_ = mine->playerID();
        
        /* Remove the mine that blew up, and terminate
           the loop so that only one mine explodes. */
        Global::mineList.erase(mine);
        break;
        
        ++mine;
      }
    }
      break;
        
        
        
    ////////////////////////// Part Event /////////////////////////////////////////////////////
        
    /** Removes the mines that a player has on the field 
        when he or she leaves.  This will ensure only valid 
        player IDs will be used. */
        
    case bz_ePlayerPartEvent:
    {
      bz_PlayerJoinPartEventData *eData = 
        dynamic_cast<bz_PlayerJoinPartEventData*>(eventData);
              
      /** Can't use standard mutating algorithms with ptr_container */
      
      Global::mineList.erase_if(std::bind2nd(equal_to<Mine, int>(), 
                                             eData->playerID));
      
    
    }
      break;
        
        
        
    ////////////////////////// Die Event ///////////////////////////////////////////////////// 
      
    /** The die event needs to be used here, in order to
        properly handle player scores and other related 
        issues.  This is necessary because it is the server
        that is actually killing, not the players.  */
      
    case bz_ePlayerDieEvent:
    {
      bz_PlayerDieEventData *eData = 
        dynamic_cast<bz_PlayerDieEventData*>(eventData);
      
      // Only interested in those who were killed by a mine
      if((eData->shotID < kMineShotIDMin) || (eData->shotID > kMineShotIDMax))
        return;
      
      eData->killerID = lastKillerID_;
      sendKilledMsg(eData->playerID, eData->killerID);
     
    }
      break;
      
      
    //////////////////////// Capture Event ///////////////////////////////////////////////////// 
      
    /** Remove mines from around a team base when a flag is 
        captured. */
      
    case bz_eCaptureEvent:
    {     
      bz_CTFCaptureEventData *eData = 
        dynamic_cast<bz_CTFCaptureEventData*>(eventData);
      
      if(!Global::spawnSafety)
        return;
      
      std::vector<TeamBase>::const_iterator begin = teamBases_.begin();
      
      float basePosition[3];
      
      while(begin != teamBases_.end()) {
        
        if(begin->team == eData->teamCapped) {
          std::copy(begin->pos, begin->pos+3, basePosition); 
          break;
        }

        ++begin;
      }
      
      float maxDistance = 1.5 * bz_getBZDBDouble("_shockOutRadius");
      
      /** Can't use remove_if with ptr_container */      
      Global::mineList.erase_if(boost::bind(distance_f<Mine, float, 3>(), 
                                            _1, 
                                            basePosition) < maxDistance);      
      
    }
      break;
      
    default:
      break;
  }
  
}




      
      
      
      
      
      
      
