/*
           Copyright Devin DeLong 2006 - 2007.
  Distributed under the Boost Software License, Version 1.0.
     (See accompanying file LICENSE_1_0.txt or copy at
           http://www.boost.org/LICENSE_1_0.txt)
 */


//local
#include "Mine.h"
#include "Global.h"
#include "utility.h"
#include "MineCommand.h"

//stl
#include <string>
#include <algorithm>

//boost
#include <boost/array.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/format.hpp>
#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/lexical_cast.hpp>



std::string MineCommand::usage_[18] = {
  "/mine <sw, sr, cr, sb, bf>",
  "",
  " sw - Shock wave mine.",
  " cr - Fast bullet mine (cylindrical pattern)",
  " sr - Fast bullet mine (spherical pattern)",
  " bf - Bad flag mine (gives a bad flag)",
  " sb - Super bullet mine (Spherical pattern)",
  "",
  "Options:",
  " show - The server sends information about the mines currently on the map",
  "        (position, type, player ID)",
  " count - Shows the number of mines on the field",
  "",
  " set <variable> <value> (not implemented yet)",
  "",
  "   Variables:",
  "     spawnSafety (bool) - If true, removes mines from around a base on flag capture",
  "     teamKillSafety (bool) - If true, teammates cannot cause mines to blow up"
};



bool MineCommand::handle (int playerID, 
                          bzApiString /*command*/, 
                          bzApiString /*message*/, 
                          bzAPIStringList* params)
{
  this->player_ = bz_getPlayerByIndex(playerID);
  this->playerID_ = playerID;
  
  if(!player_)
    return true;
  
  bzAPIStringList& args = *params;
  
  if(args.size() == 0) {
    sendUsage();
    return true;
  }
  
  std::string param = args[0].c_str();
  boost::to_upper(param);
  
  static const std::string needUselessFlag = 
    "You need to have the USeless flag to lay a mine.";
  
  if(param == "SW") {
    
    if(playerHasUS()) 
      addMine(eShockWaveMine);  
    else
      bz_sendTextMessage(BZ_SERVER, playerID, needUselessFlag.c_str());
    
  } else if(param == "SR") {
    
    if(playerHasUS()) 
      addMine(eRegularSphereMine);  
    else
      bz_sendTextMessage(BZ_SERVER, playerID, needUselessFlag.c_str());    
  
  } else if(param == "SB") {
    
    if(playerHasUS()) 
      addMine(eSuperBulletMine);  
    else
      bz_sendTextMessage(BZ_SERVER, playerID, needUselessFlag.c_str());
    
  } else if(param == "CR") {
    
    if(playerHasUS()) 
      addMine(eRegularCylinderMine);  
    else
      bz_sendTextMessage(BZ_SERVER, playerID, needUselessFlag.c_str());    
 
  } else if(param == "BF") {
    
    if(playerHasUS()) 
      addMine(eBadFlagMine);  
    else
      bz_sendTextMessage(BZ_SERVER, playerID, needUselessFlag.c_str());
    
  } else if(param == "SHOW") {
    
    if(player_) {
      if(player_->admin)
        showMines();
      else
        bz_sendTextMessage(BZ_SERVER, playerID, 
                           "You do not have permission to use this command.");
    }
    
  } else if(param == "SET") {
    
    // TODO: /mine set <variable> <value>
    
  } else if(param == "TEST") {
    
    // I don't remember
    
  } else if(param == "COUNT") {
    
    std::string msg = "There are currently ";
    msg += boost::lexical_cast<std::string>(Global::mineList.size());
    msg += " mines on the field.";
    
    bz_sendTextMessage(BZ_SERVER, playerID_, msg.c_str());
    
    boost::ptr_vector<Mine>::iterator begin = Global::mineList.begin();
    boost::ptr_vector<Mine>::iterator end = Global::mineList.end();
    
    std::vector<int> playerIDs;
        
    
    // This loop collects all of the player IDs of the players
    // who have mines on the field.
    while(begin != end) {
      
      bool playerIDExists = false;
      
      if(playerIDs.size() == 0) {
        playerIDs.push_back(begin->playerID());
        ++begin;
        continue;        
      }
      
      for(std::size_t i = 0; i < playerIDs.size(); ++i) {
        
        if(begin->playerID() == playerIDs[i]) { 
          playerIDExists = true; 
          break;
        }                
      }
      
      if(!playerIDExists) 
        playerIDs.push_back(begin->playerID());
      
      ++begin;      
    } 
    
    
    // Yet another loop.
    // Sends the number of mines each person has on the field
    for(std::size_t i = 0; i < playerIDs.size(); ++i) {
      
      std::size_t size = 
        std::count_if(Global::mineList.begin(),
                      Global::mineList.end(),
                      std::bind2nd(equal_to<Mine, int>(), playerIDs[i]));
      
      bz_PlayerRecord *player = bz_getPlayerByIndex(playerIDs[i]);
      if(!player) continue;
      
      std::string currentMsg = player->callsign.c_str();
      currentMsg += " : ";
      currentMsg += boost::lexical_cast<std::string>(size);
      
      bz_sendTextMessage(BZ_SERVER, playerID_, currentMsg.c_str());
      
      bz_freePlayerRecord(player);
    }
    
  }
  
  bz_freePlayerRecord(player_);
  return true;
}


void MineCommand::showMines() const
{
  // Format: "(x, y, z) : SR : player #13"
    
    boost::ptr_vector<Mine>::iterator mine = Global::mineList.begin();
    boost::ptr_vector<Mine>::iterator end = Global::mineList.end();
    
    boost::format fmt("(%1%, %2%, %3%) : %4% : player #%5%");
    
    while(mine != end) {
    
    float pos[3];     
    std::copy(mine->position().begin(), 
              mine->position().end(), 
              pos);
    
    std::string type;
    switch(mine->type()) {
     
      case eShockWaveMine:
        type = "SW";
        break;
      case eRegularSphereMine:
        type = "SR";
        break;
      case eRegularCylinderMine:
        type = "CR";
        break;
      case eSuperBulletMine:
        type = "SB";
        break;
      case eBadFlagMine:
        type = "BF";
        break;
      case eNullMine:
        break;
    }
    
    std::string msg = boost::str(fmt % pos[0] % pos[1] % pos[2] % type % mine->playerID());
    
    if(player_)
      bz_sendTextMessage(BZ_SERVER, player_->playerID, msg.c_str());
    
    ++mine;
  }  
}


bool MineCommand::playerHasUS() const
{
  if(!player_) 
    return false;
  
  std::string flag = std::string(player_->currentFlag.c_str());
  
  if(flag.find("US") == std::string::npos)
    return false;
  else 
    return true;
}


bool MineCommand::addMine(eMineType type) const
{
  if(!this->player_) 
    return false;
  
  boost::array<float, 3>  pos;
  std::copy(player_->pos, player_->pos + 3, pos.begin());  
  pos[2] += (float)bz_getBZDBDouble("_tankHeight")/2;
  
  switch(type) {      
    
    case eShockWaveMine:
      Global::mineList.push_back(new SWMine(playerID_,  pos));      
      break;
    
    case eRegularSphereMine:
      Global::mineList.push_back(new SRMine(playerID_,  pos));      
      break;
    
    case eRegularCylinderMine:
      Global::mineList.push_back(new CRMine(playerID_,  pos));      
      break;
    
    case eSuperBulletMine:
      Global::mineList.push_back(new SBMine(playerID_,  pos));     
      break;
    
    case eBadFlagMine:
      Global::mineList.push_back(new BFMine(playerID_,  pos));      
      break;
    
    case eNullMine:
      break;
  }
  
  bz_removePlayerFlag(playerID_);
  
  return true;
}


bool MineCommand::sendUsage() const
{
  if(!player_)
    return false;
  
  std::size_t i = 0;
 
  while(i < 18) {
    bz_sendTextMessage(BZ_SERVER, player_->playerID, usage_[i].c_str());
    ++i;
  }
  
  return true;
}










