/*
 *          Copyright Devin DeLong 2006 - 2007.
 * Distributed under the Boost Software License, Version 1.0.
 *    (See accompanying file LICENSE_1_0.txt or copy at
 *          http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef UTILITY_H
#define UTILITY_H

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// Boost
#include <boost/array.hpp>
#include <boost/format.hpp>

// STL
#include <string>
#include <functional>
#include <algorithm>
#include <vector>
#include <cmath>

// Local
#include "Mine.h"

/** Used to square and sum all of the components of a vector
    of N dimensions, as long as N is relatively small. */
template <int N, typename T> class magnitude_squared {

public:

  static T algorithm(T const *a, T const *b)
  {
    return (*a - *b) * (*a - *b) + magnitude_squared<N-1, T>::algorithm(a+1, b+1);
  }
  
  static T algorithm(T const *a)
  {
    return (*a) * (*a) + magnitude_squared<N-1, T>::algorithm(a+1);
  }
  
};

template <typename T> class magnitude_squared<1, T> {

public:

  static T algorithm(T const *a, T const *b) 
  {
    return (*a - *b) * (*a - *b);
  }
  
  static T algorithm(T const *a)
  {
    return (*a) * (*a);
  }
  
};


/** Do not define because, for the purposes of this plugin, 
    it is too much work to determine what the return type 
    should be */ 
template <class T, class U, std::size_t N>
class distance_f;


template <>
class distance_f<Mine, float, 3> : public std::binary_function<Mine, float, float> {
  
public:
  
  float operator()(const Mine& p1, float const * p2) const
  {
    return std::sqrt(magnitude_squared<3, float>::algorithm(p1.position().data(), p2));
  }
};


template <class T, std::size_t N>
class distance_f<T, T, N> : public std::binary_function<T, T, T> {
  
public:
  
  T operator()(T const * p1, T const * p2) const
  {
     return std::sqrt(magnitude_squared<N, T>::algorithm(p1, p2));
  }
};


template <class T, std::size_t N>
class distance_f<
                 boost::array<T, N>, 
                 boost::array<T, N>, 
                 N
                > 
: public std::binary_function<
                              boost::array<T, N>, 
                              boost::array<T, N>, 
                              T
                              > {
  
public:
  
  T operator()(const boost::array<T, N>& p1, 
               const boost::array<T, N>& p2) const
  {
    return std::sqrt(magnitude_squared<N, T>::algorithm(p1.data(), p2.data()));
  }
};



template <class T, class U>
class equal_to : public std::binary_function<T, U, bool> {
  
public:
  
  bool operator()(const T& t, const U& u) const
  {
    return t == u;
  }
  
};


/** Represents a team base. */

struct TeamBase {
  
  TeamBase(float const *pos, bz_eTeamType team)
  {
    this->pos[0] = pos[0]; 
    this->pos[1] = pos[1];
    this->pos[2] = pos[2];
    
    this->team = team;
  }
  
  float pos[3];
  bz_eTeamType team;
};


/** A function to give a bad flag */

void giveBadFlag(int playerID);

/** Fills the vector with all of the team bases on the map. */

void getTeamBases(std::vector<TeamBase>& teamBases);

/** Returns distances */

float distance3f(const Mine& mine, float const* position);
float distance3f(const boost::array<float, 3>& p1, const boost::array<float, 3>& p2);
float distance3f(float const* p1, float const* p2);
float distance2f(const Mine& mine, float const* position);
float distance2f(const boost::array<float, 2>& p1, const boost::array<float, 2>& p2);
float distance2f(float const* p1, float const* p2);


/** Used in place of the bz function
 
    A little more expensive in performance, but the 
    benefits of using std::vector over bzAPIIntList
    justify use of the function. 
 */

void getPlayerIndexList(const std::vector<int>& newList);

/** bz container classes are merely intended for transporting
    data between the application and a dll on the Windows 
    platform.  Thus, this function helps get the data from 
    the transport medium and into a better container. */
void toSTLVector(std::vector<std::string>& newVec,
                 bzAPIStringList& oldVec);

/** Simple way to use std::string, rather than bzApiString */

std::string toSTLString(const bzApiString& string);

void sendKilledMsg(int victimID, int killerID);



#endif /* UTILITY_H */















